@@ -36,6 +36,8 @@
 #include <sys/time.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
+#include <netinet/tcp.h>
 #include "usbredirhost.h"
 
 
@@ -49,6 +51,9 @@ static struct usbredirhost *host;
 static const struct option longopts[] = {
     { "port", required_argument, NULL, 'p' },
     { "verbose", required_argument, NULL, 'v' },
+    { "ipv4", required_argument, NULL, '4' },
+    { "ipv6", required_argument, NULL, '6' },
+    { "keepalive", required_argument, NULL, 'k' },
     { "help", no_argument, NULL, 'h' },
     { NULL, 0, NULL, 0 }
 };
@@ -93,7 +98,10 @@ static int usbredirserver_write(void *priv, uint8_t *data, int count)
 static void usage(int exit_code, char *argv0)
 {
     fprintf(exit_code? stderr:stdout,
-        "Usage: %s [-p|--port <port>] [-v|--verbose <0-5>] <usbbus-usbaddr|vendorid:prodid>\n",
+        "Usage: %s [-p|--port <port>] [-v|--verbose <0-5>] "
+        "[[-4|--ipv4 ipaddr]|[-6|--ipv6 ipaddr]] "
+        "[-k|--keepalive seconds] "
+        "<busnum-devnum|vendorid:prodid>\n",
         argv0);
     exit(exit_code);
 }
@@ -198,11 +206,16 @@ int main(int argc, char *argv[])
     int usbvendor  = -1;
     int usbproduct = -1;
     int on = 1;
-    struct sockaddr_in6 serveraddr;
+    int keepalive  = -1;
+    char *ipv4_addr = NULL, *ipv6_addr = NULL;
+    union {
+        struct sockaddr_in v4;
+        struct sockaddr_in6 v6;
+    } serveraddr;
     struct sigaction act;
     libusb_device_handle *handle = NULL;
 
-    while ((o = getopt_long(argc, argv, "hp:v:", longopts, NULL)) != -1) {
+    while ((o = getopt_long(argc, argv, "hp:v:4:6:k:", longopts, NULL)) != -1) {
         switch (o) {
         case 'p':
             port = strtol(optarg, &endptr, 10);
@@ -218,6 +231,19 @@ int main(int argc, char *argv[])
                 usage(1, argv[0]);
             }
             break;
+        case '4':
+            ipv4_addr = optarg;
+            break;
+        case '6':
+            ipv6_addr = optarg;
+            break;
+        case 'k':
+            keepalive = strtol(optarg, &endptr, 10);
+            if (*endptr != '\0') {
+                fprintf(stderr, "Invalid value for -k: '%s'\n", optarg);
+                usage(1, argv[0]);
+            }
+            break;
         case '?':
         case 'h':
             usage(o == '?', argv[0]);
@@ -244,11 +270,12 @@ int main(int argc, char *argv[])
             invalid_usb_device_id(argv[optind], argv[0]);
         }
         usbvendor = strtol(argv[optind], &endptr, 16);
-        if (*endptr != ':') {
+        if (*endptr != ':' || usbvendor <= 0 || usbvendor > 0xffff) {
             invalid_usb_device_id(argv[optind], argv[0]);
         }
         usbproduct = strtol(delim + 1, &endptr, 16);
-        if (*endptr != '\0') {
+        /* Product ID 0000 is valid */
+        if (*endptr != '\0' || usbproduct < 0 || usbproduct > 0xffff) {
             invalid_usb_device_id(argv[optind], argv[0]);
         }
     }
@@ -270,11 +297,19 @@ int main(int argc, char *argv[])
         exit(1);
     }
 
+#if LIBUSB_API_VERSION >= 0x01000106
+    libusb_set_option(ctx, LIBUSB_OPTION_LOG_LEVEL, verbose);
+#else
     libusb_set_debug(ctx, verbose);
+#endif
 
-    server_fd = socket(AF_INET6, SOCK_STREAM, 0);
+    if (ipv4_addr) {
+        server_fd = socket(AF_INET, SOCK_STREAM, 0);
+    } else {
+        server_fd = socket(AF_INET6, SOCK_STREAM, 0);
+    }
     if (server_fd == -1) {
-        perror("Error creating ipv6 socket");
+        perror("Error creating ip socket");
         exit(1);
     }
 
@@ -282,14 +317,34 @@ int main(int argc, char *argv[])
         perror("Error setsockopt(SO_REUSEADDR) failed");
         exit(1);
     }
-                                                      
-    memset(&serveraddr, 0, sizeof(serveraddr));
-    serveraddr.sin6_family = AF_INET6;
-    serveraddr.sin6_port   = htons(port);
-    serveraddr.sin6_addr   = in6addr_any;
 
-    if (bind(server_fd, (struct sockaddr *)&serveraddr, sizeof(serveraddr))) {
-        fprintf(stderr, "Error binding port %d: %s\n", port, strerror(errno));
+    memset(&serveraddr, 0, sizeof(serveraddr));
+
+    if (ipv4_addr) {
+        serveraddr.v4.sin_family = AF_INET;
+        serveraddr.v4.sin_port   = htons(port);
+        if ((inet_pton(AF_INET, ipv4_addr,
+                       &serveraddr.v4.sin_addr)) != 1) {
+            perror("Error convert ipv4 address");
+            exit(1);
+        }
+    } else {
+        serveraddr.v6.sin6_family = AF_INET6;
+        serveraddr.v6.sin6_port   = htons(port);
+        if (ipv6_addr) {
+            if ((inet_pton(AF_INET6, ipv6_addr,
+                           &serveraddr.v6.sin6_addr)) != 1) {
+                perror("Error convert ipv6 address");
+                exit(1);
+            }
+        } else {
+            serveraddr.v6.sin6_addr   = in6addr_any;
+        }
+    }
+
+    if (bind(server_fd, (struct sockaddr *)&serveraddr,
+             sizeof(serveraddr))) {
+        perror("Error bind");
         exit(1);
     }
 
@@ -308,6 +363,38 @@ int main(int argc, char *argv[])
             break;
         }
 
+        if (keepalive > 0) {
+            int optval = 1;
+            socklen_t optlen = sizeof(optval);
+            if (setsockopt(client_fd, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen) == -1) {
+                if (errno != ENOTSUP) {
+                    perror("setsockopt SO_KEEPALIVE error.");
+                    break;
+                }
+            }
+            optval = keepalive;	/* set default TCP_KEEPIDLE time from cmdline */
+            if (setsockopt(client_fd, SOL_TCP, TCP_KEEPIDLE, &optval, optlen) == -1) {
+                if (errno != ENOTSUP) {
+                    perror("setsockopt TCP_KEEPIDLE error.");
+                    break;
+                }
+            }
+            optval = 10;	/* set default TCP_KEEPINTVL time as 10s */
+            if (setsockopt(client_fd, SOL_TCP, TCP_KEEPINTVL, &optval, optlen) == -1) {
+                if (errno != ENOTSUP) {
+                    perror("setsockopt TCP_KEEPINTVL error.");
+                    break;
+                }
+            }
+            optval = 3;	/* set default TCP_KEEPCNT as 3 */
+            if (setsockopt(client_fd, SOL_TCP, TCP_KEEPCNT, &optval, optlen) == -1) {
+                if (errno != ENOTSUP) {
+                    perror("setsockopt TCP_KEEPCNT error.");
+                    break;
+                }
+            }
+        }
+
         flags = fcntl(client_fd, F_GETFL);
         if (flags == -1) {
             perror("fcntl F_GETFL");
@@ -327,6 +414,14 @@ int main(int argc, char *argv[])
                 fprintf(stderr,
                     "Could not open an usb-device with vid:pid %04x:%04x\n",
                     usbvendor, usbproduct);
+            } else if (verbose >= usbredirparser_info) {
+                libusb_device *dev;
+                dev = libusb_get_device(handle);
+                fprintf(stderr, "Open a usb-device with vid:pid %04x:%04x on "
+                        "bus %03x device %03x\n",
+                        usbvendor, usbproduct,
+                        libusb_get_bus_number(dev),
+                        libusb_get_device_address(dev));
             }
         } else {
             libusb_device **list = NULL;
@@ -341,12 +436,12 @@ int main(int argc, char *argv[])
             if (i < n) {
                 if (libusb_open(list[i], &handle) != 0) {
                     fprintf(stderr,
-                        "Could not open usb-device at bus-addr %d-%d\n",
+                        "Could not open usb-device at busnum-devnum %d-%d\n",
                         usbbus, usbaddr);
                 }
             } else {
                 fprintf(stderr,
-                    "Could not find an usb-device at bus-addr %d-%d\n",
+                    "Could not find an usb-device at busnum-devnum %d-%d\n",
                     usbbus, usbaddr);
             }
             libusb_free_device_list(list, 1);
